package com.wechi.adweb.bridge.google.analytics.dto.report.row;

import static com.wechi.adweb.bridge.google.analytics.dto.report.ReportConstant.*;

import com.google.analytics.data.v1beta.Row;
import com.wechi.adweb.bridge.google.analytics.dto.report.GAReportRequestDTO;
import com.wechi.adweb.bridge.util.NumberUtils;

import lombok.Builder;
import lombok.Data;

import java.util.List;

/**
 * @author wfansh
 */
@Data
@Builder
public class SourceMediaViewRow implements GAReportRowDTO {
    private String type;

    private int totalUsers;

    private int newUsers;

    private int sessions;

    private String bounceRate;

    private String avgSessionDuration;

    private String pageViewsPerSession;

    public static List<SourceMediaViewRow> fromReportRows(
            List<Row> reportRows, GAReportRequestDTO reportRequest) {
        int indexType = reportRequest.getDimensions().indexOf(DIMENSION_SESSION_SOURCE_MEDIUM);
        int indexTotalUsers = reportRequest.getMetrics().indexOf(METRIC_TOTAL_USERS);
        int indexNewUsers = reportRequest.getMetrics().indexOf(METRIC_NEW_USERS);
        int indexSessions = reportRequest.getMetrics().indexOf(METRIC_SESSIONS);
        int indexBounceRate = reportRequest.getMetrics().indexOf(METRIC_BOUNCE_RATE);
        int indexAvgSessionDuration =
                reportRequest.getMetrics().indexOf(METRIC_AVG_SESSION_DURATION);
        int indexPageViewsPerSession =
                reportRequest.getMetrics().indexOf(METRIC_SCREEN_PAGE_VIEWS_PER_SESSION);

        return reportRows.stream()
                .map(
                        row ->
                                SourceMediaViewRow.builder()
                                        .type(row.getDimensionValues(indexType).getValue())
                                        .totalUsers(
                                                Integer.parseInt(
                                                        row.getMetricValues(indexTotalUsers)
                                                                .getValue()))
                                        .newUsers(
                                                Integer.parseInt(
                                                        row.getMetricValues(indexNewUsers)
                                                                .getValue()))
                                        .sessions(
                                                Integer.parseInt(
                                                        row.getMetricValues(indexSessions)
                                                                .getValue()))
                                        .bounceRate(
                                                NumberUtils.formatPercentage(
                                                        Double.parseDouble(
                                                                row.getMetricValues(indexBounceRate)
                                                                        .getValue()),
                                                        2))
                                        .avgSessionDuration(
                                                NumberUtils.formatDecimal(
                                                        Double.parseDouble(
                                                                row.getMetricValues(
                                                                                indexAvgSessionDuration)
                                                                        .getValue()),
                                                        2))
                                        .pageViewsPerSession(
                                                NumberUtils.formatDecimal(
                                                        Double.parseDouble(
                                                                row.getMetricValues(
                                                                                indexPageViewsPerSession)
                                                                        .getValue()),
                                                        2))
                                        .build())
                .toList();
    }
}
